@@ -16,6 +16,7 @@
 --
 
 local file = require("apisix.cli.file")
+local schema = require("apisix.cli.schema")
 
 
 local _M = {}
@@ -39,6 +40,9 @@ function _M.local_conf(force)
         return nil, err
     end
 
+    -- fill the default value by the schema
+    schema.validate(default_conf)
+
     config_data = default_conf
     return config_data
 end
