@@ -143,6 +143,18 @@ do
         var_x_forwarded_proto = true,
     }
 
+    -- sort in alphabetical
+    local apisix_var_names = {
+        balancer_ip = true,
+        balancer_port = true,
+        consumer_name = true,
+        mqtt_client_id = true,
+        route_id = true,
+        route_name = true,
+        service_id = true,
+        service_name = true,
+    }
+
     local mt = {
         __index = function(t, key)
             local cached = t._cache[key]
@@ -205,29 +217,19 @@ do
                 key = sub_str(key, 9)
                 val = get_parsed_graphql()[key]
 
-            elseif key == "route_id" then
-                val = ngx.ctx.api_ctx and ngx.ctx.api_ctx.route_id
-
-            elseif key == "service_id" then
-                val = ngx.ctx.api_ctx and ngx.ctx.api_ctx.service_id
-
-            elseif key == "consumer_name" then
-                val = ngx.ctx.api_ctx and ngx.ctx.api_ctx.consumer_name
-
-            elseif key == "route_name" then
-                val = ngx.ctx.api_ctx and ngx.ctx.api_ctx.route_name
-
-            elseif key == "service_name" then
-                val = ngx.ctx.api_ctx and ngx.ctx.api_ctx.service_name
-
-            elseif key == "balancer_ip" then
-                val = ngx.ctx.api_ctx and ngx.ctx.api_ctx.balancer_ip
-
-            elseif key == "balancer_port" then
-                val = ngx.ctx.api_ctx and ngx.ctx.api_ctx.balancer_port
-
             else
-                val = get_var(key, t._request)
+                local getter = apisix_var_names[key]
+                if getter then
+                    if getter == true then
+                        val = ngx.ctx.api_ctx and ngx.ctx.api_ctx[key]
+                    else
+                        -- the getter is registered by ctx.register_var
+                        val = getter(ngx.ctx.api_ctx)
+                    end
+
+                else
+                    val = get_var(key, t._request)
+                end
             end
 
             if val ~= nil and not no_cacheable_var_names[key] then
@@ -247,6 +249,18 @@ do
         end,
     }
 
+function _M.register_var(name, getter)
+    if type(getter) ~= "function" then
+        error("the getter of registered var should be a function")
+    end
+
+    if apisix_var_names[name] then
+        error(name .. " is registered")
+    end
+
+    apisix_var_names[name] = getter
+end
+
 function _M.set_vars_meta(ctx)
     local var = tablepool.fetch("ctx_var", 0, 32)
     if not var._cache then
