@@ -16,6 +16,11 @@
 --
 local limit_local_new = require("resty.limit.count").new
 local core = require("apisix.core")
+local tab_insert = table.insert
+local ipairs = ipairs
+local pairs = pairs
+
+
 local plugin_name = "limit-count"
 local limit_redis_cluster_new
 local limit_redis_new
@@ -29,16 +34,63 @@ end
 local lrucache = core.lrucache.new({
     type = 'plugin', serial_creating = true,
 })
+local group_conf_lru = core.lrucache.new({
+    type = 'plugin',
+})
 
 
+local policy_to_additional_properties = {
+    redis = {
+        properties = {
+            redis_host = {
+                type = "string", minLength = 2
+            },
+            redis_port = {
+                type = "integer", minimum = 1, default = 6379,
+            },
+            redis_password = {
+                type = "string", minLength = 0,
+            },
+            redis_database = {
+                type = "integer", minimum = 0, default = 0,
+            },
+            redis_timeout = {
+                type = "integer", minimum = 1, default = 1000,
+            },
+        },
+        required = {"redis_host"},
+    },
+    ["redis-cluster"] = {
+        properties = {
+            redis_cluster_nodes = {
+                type = "array",
+                minItems = 2,
+                items = {
+                    type = "string", minLength = 2, maxLength = 100
+                },
+            },
+            redis_password = {
+                type = "string", minLength = 0,
+            },
+            redis_timeout = {
+                type = "integer", minimum = 1, default = 1000,
+            },
+            redis_cluster_name = {
+                type = "string",
+            },
+        },
+        required = {"redis_cluster_nodes", "redis_cluster_name"},
+    },
+}
 local schema = {
     type = "object",
     properties = {
         count = {type = "integer", exclusiveMinimum = 0},
         time_window = {type = "integer",  exclusiveMinimum = 0},
+        group = {type = "string"},
         key = {type = "string", default = "remote_addr"},
         key_type = {type = "string",
-            enum = {"var", "var_combination"},
+            enum = {"var", "var_combination", "constant"},
             default = "var",
         },
         rejected_code = {
@@ -66,53 +118,20 @@ local schema = {
                         },
                     },
                 },
-                {
+                core.table.merge({
                     properties = {
                         policy = {
                             enum = {"redis"},
                         },
-                        redis_host = {
-                            type = "string", minLength = 2
-                        },
-                        redis_port = {
-                            type = "integer", minimum = 1, default = 6379,
-                        },
-                        redis_password = {
-                            type = "string", minLength = 0,
-                        },
-                        redis_database = {
-                            type = "integer", minimum = 0, default = 0,
-                        },
-                        redis_timeout = {
-                            type = "integer", minimum = 1, default = 1000,
-                        },
                     },
-                    required = {"redis_host"},
-                },
-                {
+                }, policy_to_additional_properties.redis),
+                core.table.merge({
                     properties = {
                         policy = {
                             enum = {"redis-cluster"},
                         },
-                        redis_cluster_nodes = {
-                            type = "array",
-                            minItems = 2,
-                            items = {
-                                type = "string", minLength = 2, maxLength = 100
-                            },
-                        },
-                        redis_password = {
-                            type = "string", minLength = 0,
-                        },
-                        redis_timeout = {
-                            type = "integer", minimum = 1, default = 1000,
-                        },
-                        redis_cluster_name = {
-                            type = "string",
-                        },
                     },
-                    required = {"redis_cluster_nodes", "redis_cluster_name"},
-                }
+                }, policy_to_additional_properties["redis-cluster"]),
             }
         }
     }
@@ -127,12 +146,42 @@ local _M = {
 }
 
 
+local function group_conf(conf)
+    return conf
+end
+
+
 function _M.check_schema(conf)
     local ok, err = core.schema.check(schema, conf)
     if not ok then
         return false, err
     end
 
+    if conf.group then
+        local fields = {}
+        for k in pairs(schema.properties) do
+            tab_insert(fields, k)
+        end
+        local extra = policy_to_additional_properties[conf.policy]
+        if extra then
+            for k in pairs(extra.properties) do
+                tab_insert(fields, k)
+            end
+        end
+
+        local prev_conf = group_conf_lru(conf.group, "", group_conf, conf)
+
+        for _, field in ipairs(fields) do
+            if not core.table.deep_eq(prev_conf[field], conf[field]) then
+                core.log.error("previous limit-conn group ", prev_conf.group,
+                            " conf: ", core.json.encode(prev_conf))
+                core.log.error("current limit-conn group ", conf.group,
+                            " conf: ", core.json.encode(conf))
+                return false, "group conf mismatched"
+            end
+        end
+    end
+
     return true
 end
 
@@ -161,7 +210,14 @@ end
 
 function _M.access(conf, ctx)
     core.log.info("ver: ", ctx.conf_version)
-    local lim, err = core.lrucache.plugin_ctx(lrucache, ctx, conf.policy, create_limit_obj, conf)
+
+    local lim, err
+    if not conf.group then
+        lim, err = core.lrucache.plugin_ctx(lrucache, ctx, conf.policy, create_limit_obj, conf)
+    else
+        lim, err = lrucache(conf.group, "", create_limit_obj, conf)
+    end
+
     if not lim then
         core.log.error("failed to fetch limit.count object: ", err)
         if conf.allow_degradation then
@@ -182,6 +238,8 @@ function _M.access(conf, ctx)
         if n_resolved == 0 then
             key = nil
         end
+    elseif conf.key_type == "constant" then
+        key = conf_key
     else
         key = ctx.var[conf_key]
     end
@@ -192,7 +250,13 @@ function _M.access(conf, ctx)
         key = ctx.var["remote_addr"]
     end
 
-    key = key .. ctx.conf_type .. ctx.conf_version
+    -- here we add a separator ':' to mark the boundary of the prefix and the key itself
+    if not conf.group then
+        key = ctx.conf_type .. ctx.conf_version .. ':' .. key
+    else
+        key = conf.group .. ':' .. key
+    end
+
     core.log.info("limit key: ", key)
 
     local delay, remaining = lim:incoming(key, true)
