@@ -42,6 +42,7 @@ local schema = {
         batch_max_size = {type = "integer", minimum = 1, default= 1000},
     }
 }
+batch_processor.schema = schema
 
 
 local function schedule_func_exec(self, delay, batch)
@@ -126,7 +127,7 @@ function batch_processor:new(func, config)
         return nil, err
     end
 
-    if type(func) ~= "function" then
+    if not(type(func) == "function") then
         return nil, "Invalid argument, arg #1 must be a function"
     end
 
